!########################################################################!

 MODULE const_and_precisions
!########################################################################!
 IMPLICIT NONE
 PUBLIC

!------------------------------------------------------------------------
! common precisions
!------------------------------------------------------------------------

!  INTEGER, PARAMETER :: I1 = SELECTED_INT_KIND (2)        ! Integer*1
!  INTEGER, PARAMETER :: I2 = SELECTED_INT_KIND (4)        ! Integer*2
  INTEGER, PARAMETER :: I4 = SELECTED_INT_KIND (9)        ! Integer*4
!  INTEGER, PARAMETER :: I8 = SELECTED_INT_KIND (18)       ! Integer*8
!  INTEGER, PARAMETER :: R4 = SELECTED_REAL_KIND (6, 37)   ! Real*4
  INTEGER, PARAMETER :: R8 = SELECTED_REAL_KIND (15, 300) ! Real*8
  INTEGER, PARAMETER :: wp_   = R8        ! work-precision
! INTEGER, PARAMETER :: odep_ = R8        ! ODE-solver precision
! INTEGER, PARAMETER :: xp_   = wp_       ! for ext. modules if necessary
!------------------------------------------------------------------------
! precisions which are in use in CONFIG_yat
!------------------------------------------------------------------------
! INTEGER, PARAMETER :: ypi_ = 4          ! <- direct precision def.
! INTEGER, PARAMETER :: ypd_ = 8          ! <- direct precision def.
!------------------------------------------------------------------------
! length of the file names
!------------------------------------------------------------------------
 !INTEGER, PARAMETER :: lfn_ = 256        ! <- requested for yat
!!========================================================================
! Arithmetic constants
!========================================================================
  REAL(wp_), PARAMETER :: zero    = 0.0_wp_
  REAL(wp_), PARAMETER :: unit    = 1.0_wp_
  real (kind = R8), parameter :: pi = 3.141592653589793238462643383280_R8
! REAL(wp_), PARAMETER :: sqrt_pi = 1.772453850905516_wp_
! REAL(wp_), PARAMETER :: sqrt_2  = 1.414213562373095_wp_
! REAL(wp_), PARAMETER :: rad     = pi/180.0_wp_
!---
! REAL(wp_), PARAMETER :: ex(1:3) = (/unit,zero,zero/)
! REAL(wp_), PARAMETER :: ey(1:3) = (/zero,unit,zero/)
! REAL(wp_), PARAMETER :: ez(1:3) = (/zero,zero,unit/)
!---
! REAL(wp_), PARAMETER :: kron(3,3) = reshape((/unit,zero,zero, &
!                                               zero,unit,zero, &
!                                               zero,zero,unit/),(/3,3/))
! COMPLEX(wp_), PARAMETER :: im    = (0.0_wp_,1.0_wp_)
! COMPLEX(wp_), PARAMETER :: czero = (0.0_wp_,0.0_wp_)
! COMPLEX(wp_), PARAMETER :: cunit = (1.0_wp_,0.0_wp_)
! COMPLEX(wp_), PARAMETER :: ctwo  = (2.0_wp_,0.0_wp_)
!========================================================================
! Computer constants
!========================================================================
  REAL(kind = R8), PARAMETER :: comp_eps      = EPSILON(unit)
!  REAL(wp_), PARAMETER :: comp_eps2     = comp_eps**2
! REAL(wp_), PARAMETER :: comp_tiny     = TINY(unit)
! REAL(wp_), PARAMETER :: comp_huge     = HUGE(unit)
! REAL(wp_), PARAMETER :: comp_tinylog  =-200 ! LOG10(comp_tiny)
! REAL(wp_), PARAMETER :: comp_hugelog  =+200 ! LOG10(comp_huge)
! REAL(wp_), PARAMETER :: comp_tiny1    = 1d+50*comp_tiny
! REAL(wp_), PARAMETER :: comp_huge1    = 1d-50*comp_huge
! REAL(wp_), PARAMETER :: comp_tiny1log = LOG10(comp_tiny1)
! REAL(wp_), PARAMETER :: comp_huge1log = LOG10(comp_huge1)
!------------------------------------------------------------------------
! Conventional constants
!------------------------------------------------------------------------
! REAL(wp_), PARAMETER :: output_tiny = 1.0d-66
! REAL(wp_), PARAMETER :: output_huge = 1.0d+66
!========================================================================
! Physical constants (SI)
!========================================================================
  real (kind = R8), parameter :: e_ = 1.602176487e-19_R8  ! [C]
  real (kind = R8), parameter :: me_ = 9.10938215e-31_R8      ! electron mass [kg]
!  real (kind = R8), parameter :: mp_ = 1.672621637e-27_R8     ! proton mass [kg]
! REAL(wp_), PARAMETER :: rmpe_  = mp_/me_
!  real (kind = R8), parameter :: md_ = 3.34358320e-27_R8      ! deuteron mass, kg
!  real (kind = R8), parameter :: mt_ = 5.00735588e-27_R8      ! triton mass, kg
!  real (kind = R8), parameter :: ma_ = 6.64465620e-27_R8      ! alpha mass, kg
!  real (kind = R8), parameter :: amu_ = 1.660538782e-27_R8    ! amu, kg
  real (kind = R8), parameter :: c_ = 2.99792458e8_R8  ! speed of light [m/s]
  real (kind = R8), parameter :: mu0_ = 4.0e-7_R8 * pi
!  real (kind = R8), parameter :: eps0_ = 1.0_R8 / (mu0_ * c_ * c_)    ! [F/m]
!  real (kind = R8), parameter :: avogr_ = 6.02214179e23_R8
!  real (kind = R8), parameter :: KBolt_ = 1.3806504e-23_R8
!------------------------------------------------------------------------
! Useful definitions
!------------------------------------------------------------------------
  real (kind = R8), parameter :: ev_ = e_          ! [J]
  REAL(wp_), PARAMETER :: keV_   = 1000*ev_        ! [J]
  REAL(wp_), PARAMETER :: mc2_SI = me_*c_**2       ! [J]
  REAL(wp_), PARAMETER :: mc2_   = mc2_SI/keV_     ! [keV]
! REAL(wp_), PARAMETER :: mc_    = me_*c_          ! [kg*m/s]
! ! f_ce = fce1_*B (B in Tesla):                   !
! REAL(wp_), PARAMETER :: wce1_  = e_/me_          ! [rad/s]
! REAL(wp_), PARAMETER :: fce1_  = wce1_/(2*pi)    ! [1/s]
! ! f_pl = fpe1_*sqrt(Ne) (Ne in 1/m**3):          !
! REAL(wp_), PARAMETER :: wpe1_  = 56.4049201      ! [rad/s]
! REAL(wp_), PARAMETER :: fpe1_  = wpe1_/(2*pi)    ! [1/s]
! REAL(wp_), PARAMETER :: wpe12_ = wpe1_**2        !
! ! vte  = vte1_*sqrt(Te) (Te in keV):             !
! REAL(wp_), PARAMETER :: vte1_  = 1.8755328e7     ! [m/s]
! ! je   = curr1_*sqrt(Te)*Ne  (Ne in 1/m**3):     !
! REAL(wp_), PARAMETER :: curr1_ = e_*vte1_        ! [A/m**2]
!!========================================================================
!! Upper limit for the momentum value for integration
!!========================================================================
! REAL(wp_), PARAMETER :: umax_ = 7.0d0            ! max of (p/pth)
! INTEGER,   PARAMETER :: nu_   = 700              ! size of upar-array
!========================================================================
! minimal value of Nparallel
!========================================================================
! REAL(wp_), PARAMETER :: Npar_min = 1.0d-3
!########################################################################!

 END MODULE const_and_precisions

!########################################################################!
