
!      (ijetto, mr, mz, mrd, r, z, psin, psiax, psibnd,
!     .    rax, zax, nbnd, rbnd, zbnd, nrho, psijet, f, te, dne,
!     .    zeff, qsf, nbeam, powin, alphin, betain, dpdv, jcd, pec, 
!     .    icd, ierr)

  implicit real*8 (a-h,o-z)
  parameter(pi=3.14159265358979d0)
!  input arguments
  integer :: ijetto, mr, mz, nbnd, nrho, nbeam
  real*8, DIMENSION(:), ALLOCATABLE :: r, z, psinr
  real*8, DIMENSION(:,:), ALLOCATABLE :: psin, psi
  real*8 psiax, psibnd, rax, zax
  real*8, DIMENSION(:), ALLOCATABLE :: rbnd, zbnd
  real*8, DIMENSION(:), ALLOCATABLE :: psijet, f, qsf, te, dne, zeff
  real*8, DIMENSION(:), ALLOCATABLE :: teitpl, dneitpl, zeffitpl, cte, ctdn, ctz
  real*8, DIMENSION(:), ALLOCATABLE :: powin, alphain, betain
!  do loop arguments
  real*8 :: powloop,alphaloop,betaloop
!  do loop output arguments
  real*8, DIMENSION(:), ALLOCATABLE :: dpdv, jcd
  real*8 :: pec, icd
!  gray_main output arguments
  real*8, DIMENSION(:), ALLOCATABLE :: dpdvloop, jcdloop
  real*8 :: pecloop, icdloop
  integer :: ierr
!  unused read variales
  real*8, DIMENSION(:), ALLOCATABLE :: pres,ffprim,pprim,rlim,zlim
  character(LEN=48) :: stringa
!!!!!!!!!!  VARIABILI PER TEST  !!!!!!!!!!
  real*8,dimension(2) :: alphalim, betalim
  real*8 :: delta
  integer :: alphanum, betanum, len
  character(LEN=48) :: profile, eqfile
!!!!!!!!!!  VARIABILI PER TEST  !!!!!!!!!!
!  
!  local variables
!     real*8 fgray(nrho), qgray(nrho), jcdgry(nrho), icdgry
!
!  input arguments
!
!    ijetto  Equilibrium source (1 EFIT, 2 ESCO)
!            If IJETTO=2, then PSIN values are valid only inside
!            plasma boudary (PSIN=0 outside)
!    mr      Size of flux map grid in R direction
!    mz      Size of flux map grid in Z direction
!    mrd     Leading dimension of the psin(:,:) array, mrd>mr
!    r       R coordinates of flux map grid points [m]
!    z       Z coordinates of flux map grid points [m]
!    psin    Normalised poloidal flux psin=(psi-psiax)/(psibnd-psiax)
!            on the (R, Z) grid.
!    psiax   Poloidal flux on axis [Wb rad-1]
!    psibnd  Poloidal flux on boundary [Wb rad-1]
!    rax     R coordinate of magnetic axis [m]
!    zax     Z coordinate of magnetic axis [m]
!    nbnd    Number of points in plasma boundary contour
!    rbnd    R coordinates of plasma boundary contour [m]
!    zbnd    Z coordinates of plasma boundary contour [m]
!
!    nrho    Number of points in JETTO rho grid    -
!    psijet  Normalised poloidal flux on JETTO radial grid
!    f       Poloidal current stream function f=Bphi*R on JETTO
!            radial grid [T m]
!    te      Electron temperature on JETTO radial grid [eV]
!    dne     Electron density on JETTO radial grid [m-3]
!    zeff    Effective nuclear charge Zeff on JETTO radial grid
!    qsf     Safety factor on JETTO radial grid
!
!    nbeam   number of beams injected
!    powin   Input ECRH power array [W] (powin(i) =< 0 means i-th beam is unused)
!    alphin  Beams poloidal injection angles array [rad]
!    betain  Beams toroidal injection angles array [rad]
!
!  output arguments
!
!    dpdv    Absorbed EC power density on JETTO radial grid [W m-3]
!    jcd     EC driven flux averaged current density on JETTO
!            radial grid [A m-2]
!    pec     Total absorbed EC power [W]
!    icd     Total EC driven current [A]
!    ierr    Return code. IERR>0 on error
!            ierr = 90-93: error computing integrals for current drive
!            ierr = 94: absorption coefficient alpha < 0
!            ierr = 97: parallel comp. refract. idx N//>0.99 (warning)
!            ierr = 98: parallel comp. refract. idx N//>1.05
!
!  JETTO coordinate system assumes toroidal angle increasing CW
!  in GRAY toroidal angle increases CCW --> adapt signs on input data
  
!!!!!!!!!!  VARIABILI PER TEST  !!!!!!!!!!

  allocate(powin(1),alphain(1),betain(1))
  nbeam=1

  ijetto=1
  
!  lettura gray.data
  open(99,file='gray.data')
  read(99,*) alphain(1),betain(1)
  read(99,*)
  read(99,*) powin(1)
  powin(1)=powin(1)*1.d6
  do i=1,13
    read(99,*)
  end do
  read(99,*) eqfile
  read(99,*) ipsinorm
  read(99,*) factb
  read(99,*) profile
  read(99,*)
  read(99,*) sgnbphi,sgniphi,icocos
  close(99)
  
!  lettura profili di flusso poloidale, temperatura, densità e carica nucleare
  len=length(profile)
  open(98,file=profile(1:len)//'.prf',action='read')
  read(98,*) nrho
  allocate(psijet(nrho),te(nrho),dne(nrho),zeff(nrho))
  read(98,*) psijet0,te0,dne0,zeff0
  if(psijet0.ne.0.0d0) psijet0=0.0d0
  psijet(1)=psijet0
  te(1)=te0
  dne(1)=dne0
  zeff(1)=zeff0
  do i=2,nrho
    read(98,*) psijeti,tei,dnei,zeffi !,rhoi,qsfi
    psijet(i)=psijeti
    te(i)=tei
    dne(i)=dnei
    zeff(i)=zeffi
  end do
  close(98)
  
!  lettura dati campo magnetico
  len=length(eqfile)
  open(99,file=eqfile(1:len)//'.eqdsk',action='read')

  if(icocos.gt.0) then 
   read (99,'(a48,3i4)') stringa,idum,mr,mz
  else
   read (99,*) mr,mz
  end if
  
  allocate(r(mr),z(mz),psinr(mr),psi(mr,mz),psin(mr,mz),f(mr),qsf(mr))
  allocate(pres(mr),ffprim(mr),pprim(mr))
  allocate(dpdv(mr),jcd(mr),dpdvloop(mr), jcdloop(mr))
  
  if(icocos.gt.0) then 
   read (99,'(5e16.9)') drnr1,dznz1,rcen,rleft,zmid
   read (99,'(5e16.9)') rax,zax,psiax,psibnd,btrcen
   read (99,'(5e16.9)') current,xdum,xdum,xdum,xdum
   read (99,'(5e16.9)') xdum,xdum,xdum,xdum,xdum
   read (99,'(5e16.9)') (f(i),i=1,mr)
   read (99,'(5e16.9)') (pres(i),i=1,mr)
   read (99,'(5e16.9)') (ffprim(i),i=1,mr)
   read (99,'(5e16.9)') (pprim(i),i=1,mr)
   if (ipsinorm.eq.0) then
     read (99,'(5e16.9)') ((psi(i,j),i=1,mr),j=1,mz)
   else
     read (99,'(5e16.9)') ((psin(i,j),i=1,mr),j=1,mz)
   end if    
   read (99,'(5e16.9)') (qsf(i),i=1,mr)
   read (99,*) nbnd,nlim
   allocate(rbnd(nbnd),zbnd(nbnd),rlim(nlim),zlim(nlim))
   read (99,'(5e16.9)') (rbnd(i),zbnd(i),i=1,nbnd)
   read (99,'(5e16.9)') (rlim(i),zlim(i),i=1,nlim)
  else
   read (99,*) drnr1,dznz1,rcen,rleft,zmid
   read (99,*) rax,zax,psiax,psibnd,btrcen
   read (99,*) current,xdum,xdum,xdum,xdum
   read (99,*) xdum,xdum,xdum,xdum,xdum
   read (99,*) (f(i),i=1,mr)
   read (99,*) (pres(i),i=1,mr)
   read (99,*) (ffprim(i),i=1,mr)
   read (99,*) (pprim(i),i=1,mr)
   if (ipsinorm.eq.0) then
     read (99,*) ((psi(i,j),i=1,mr),j=1,mz)
   else
     read (99,*) ((psin(i,j),i=1,mr),j=1,mz)
   end if    
   read (99,*) (qsf(i),i=1,mr)
   read (99,*) nbnd,nlim
   allocate(rbnd(nbnd),zbnd(nbnd),rlim(nlim),zlim(nlim))
   read (99,*) (rbnd(i),zbnd(i),i=1,nbnd)
   read (99,*) (rlim(i),zlim(i),i=1,nlim)
  end if
  close(99)
  
!  adeguamento alle convenzioni e normalizzazioni
  icocosmod=mod(icocos,10)

!  icocos mod 2 = 0: toroidal angle phi CW (opposite to gray convention)
  if (mod(icocos,2).eq.0.and.icocosmod.gt.0) then
    btrcen=-btrcen
    current=-current
    do i=1,mr
      f(i)=-f(i)
    end do
  end if

  if (icocosmod.ne.0) sgnbphi=sign(1.0d0,f(mr))
  btrcen=sign(btrcen,sgnbphi)

  do i=1,mr
    f(i)=sign(f(i),sgnbphi)*factb
  end do
  
!  icocos mod 10 = 1,4,5,8: psi increasing with CCW Ip
!  icocos mod 10 = 2,3,6,7: psi decreasing with CCW Ip
  if (icocosmod.eq.1 .or. icocosmod.eq.4 .or.icocosmod.eq.5 .or. icocosmod.eq.8) then
    psibnd=-psibnd
    psiax=-psiax
    if (ipsinorm.eq.0) then
      do j=1,mz
        do i=1,mr
          psi(i,j)=-psi(i,j)
        end do
      end do
    end if
  end if
  
  psia0=psibnd-psiax
  
!  icocos=0: adapt psi to force Ip sign, otherwise maintain psi
  if (icocosmod.ne.0) sgniphi=sign(1.0d0,-psia0)
  current=sign(current,sgniphi)
  psia=sign(psia0,-sgniphi)*factb
  
!  icocos>10: input psi is in Wb
!  icocos<10: input psi is in Wb/rad (gray convention)
  if (icocos.ge.10) psia=psia/(2.0d0*pi)
  
  if(ipsinorm.eq.0) then
    do j=1,mz
      do i=1,mr
        psin(i,j)=(psi(i,j)-psiax)/(psia0)
        psi(i,j)=psin(i,j)*psia
      enddo
    enddo
  else
    do j=1,mz
      do i=1,mr
        psi(i,j)=psin(i,j)*psia
      enddo
    enddo
  end if
  
  dr=drnr1/dble(mr-1)
  dz=dznz1/dble(mz-1)
  r(1)=rleft
  z(1)=zmid-dznz1/2.0d0
  dpsinr=1.0d0/dble(mr-1)
  
  do i=1,mr
    psinr(i)=dpsinr*(i-1)
    r(i)=r(1)+(i-1)*dr
  end do
  
  do j=1,mz
    z(j)=z(1)+(j-1)*dz
  end do
  
!  interpolazione Te, dne, Zeff (psijet -> psinr, nrho -> mr elementi)
  
  allocate(teitpl(mr),dneitpl(mr),zeffitpl(mr))
  allocate(cte(4*nrho),ctdn(4*nrho),ctz(4*nrho))
  
  iopt=0
  call difcsg(psijet,te,nrho,iopt,cte,ier)
  call difcsg(psijet,dne,nrho,iopt,ctdn,ier)
  call difcsg(psijet,zeff,nrho,iopt,ctz,ier)
  
  do i=1,mr
    call vlocate(psijet,nrho,psinr(i),k)
    k = max(1,min(k,nrho-1))
    dal = psinr(i)-psijet(k)

    teitpl(i) = fspli(cte,nrho,k,dal)*1000.d0
    dneitpl(i) = fspli(ctdn,nrho,k,dal)*1.d19
    zeffitpl(i) = fspli(ctz,nrho,k,dal)
  end do
  
!  inizializzazione variabili output
  dpdv = 0
  jcd = 0
  pec = 0
  icd = 0
  
  do j=1,nbeam
!  potenza ECRH e angoli per fascio j-esimo
    powloop = powin(j)
    alphaloop = alphain(j)
    betaloop = betain(j)
    
    call gray_main(ijetto, mr, mz, r, z, psin, 0.0d0, psia, &
     rax, zax, nbnd, rbnd, zbnd, mr, psinr, f, teitpl, dneitpl, &
     zeffitpl, qsf, j, powloop, alphaloop, betaloop, &
     dpdvloop, jcdloop, pecloop, icdloop, ierr)
    
    dpdv = dpdv + dpdvloop
    jcd = jcd + jcdloop
    pec = pec + pecloop
    icd = icd + icdloop
  end do
  
end
