      subroutine gray(ijetto, mr, mz, mrd, r, z, psin, psiax, psibnd,
     .    rax, zax, nbnd, rbnd, zbnd, nrho, psijet, f, te, dne, zeff,
     .    qsf, nbeam, powin, alphin, betain, dpdv, jcd, pec, icd, ierr)
c   input arguments
      integer ijetto, mr, mz, nbnd, nrho, nbeam
      real*8 r(mr), z(mz), psin(mrd,mz)
      real*8 psiax, psibnd, rax, zax
      real*8 rbnd(nbnd), zbnd(nbnd)
      real*8 psijet(nrho), f(nrho), qsf(nrho), te(nrho), dne(nrho)
      real*8 zeff(nrho)
      real*8 powin(nbeam), alphin(nbeam), betain(nbeam)
c   output arguments
      real*8 dpdv(nrho), jcd(nrho), pec, icd
c   gray_main output arguments
      real*8 dpdvloop(nrho), jcdloop(nrho), pecloop, icdloop
      integer ierr
c   local variables
c      real*8 fgray(nrho), qgray(nrho), jcdgry(nrho), icdgry
c
c   input arguments
c
c     ijetto  Equilibrium source (1 EFIT, 2 ESCO)
c             If IJETTO=2, then PSIN values are valid only inside
c             plasma boudary (PSIN=0 outside)
c     mr      Size of flux map grid in R direction
c     mz      Size of flux map grid in Z direction
c     mrd     Leading dimension of the psin(:,:) array, mrd>mr
c     r       R coordinates of flux map grid points [m]
c     z       Z coordinates of flux map grid points [m]
c     psin    Normalised poloidal flux psin=(psi-psiax)/(psibnd-psiax)
c             on the (R, Z) grid.
c     psiax   Poloidal flux on axis [Wb rad-1]
c     psibnd  Poloidal flux on boundary [Wb rad-1]
c     rax     R coordinate of magnetic axis [m]
c     zax     Z coordinate of magnetic axis [m]
c     nbnd    Number of points in plasma boundary contour
c     rbnd    R coordinates of plasma boundary contour [m]
c     zbnd    Z coordinates of plasma boundary contour [m]
c
c     nrho    Number of points in JETTO rho grid    -
c     psijet  Normalised poloidal flux on JETTO radial grid
c     f       Poloidal current stream function f=Bphi*R on JETTO
c             radial grid [T m]
c     te      Electron temperature on JETTO radial grid [eV]
c     dne     Electron density on JETTO radial grid [m-3]
c     zeff    Effective nuclear charge Zeff on JETTO radial grid
c     qsf     Safety factor on JETTO radial grid
c
c     nbeam   Total number of injected beams
c     powin   Input ECRH power array [W] (powin(i) =< 0 means i-th beam is unused)
c     alphin  Beams poloidal injection angles array [rad]
c     betain  Beams toroidal injection angles array [rad]
c
c   output arguments
c
c     dpdv    Absorbed EC power density on JETTO radial grid [W m-3]
c     jcd     EC driven flux averaged current density on JETTO
c             radial grid [A m-2]
c     pec     Total absorbed EC power [W]
c     icd     Total EC driven current [A]
c     ierr    Return code. IERR>0 on error
c             ierr = 90-93: error computing integrals for current drive
c             ierr = 94: absorption coefficient alpha < 0
c             ierr = 97: parallel comp. refract. idx N//>0.99 (warning)
c             ierr = 98: parallel comp. refract. idx N//>1.05
c
c   JETTO coordinate system assumes toroidal angle increasing CW
c   in GRAY toroidal angle increases CCW --> adapt signs on input data
c
c      do i=1,nrho
c        f(i)=-f(i)
c        qsf(i)=-qsf(i)
c      end do
c
c   set output variables to 0
c
      dpdv = 0.d0
      jcd = 0.d0
      pec = 0.d0
      icd = 0.d0
      
      do j=1,nbeam
c
c     call main subroutine for the j-th beam
c
        if (powin(j).gt.0.0d0) then
          call gray_main(ijetto, mr, mz, r, z, psin(1:mr,:), psiax, 
     .    psibnd, rax, zax, nbnd, rbnd, zbnd, nrho, psijet, -f, te,
     .    dne, zeff, -qsf, j, powin(j), alphin(j), betain(j),
     .    dpdvloop, jcdloop, pecloop, icdloop, ierr)
c
c     add contribution of j-th beam to the total
c
          dpdv = dpdv + dpdvloop
          jcd = jcd + jcdloop
          pec = pec + pecloop
          icd = icd + icdloop
        end if
      end do
c
c     adapt output data to JETTO convention on toroidal angle
c
      do i=1,nrho
        jcd(i)=-jcd(i)
c        f(i)=-f(i)
c        qsf(i)=-qsf(i)
      end do
      icd=-icd
c
      close(604,607,608,609,612,617,630,631,633,644,645,646,648)
c
      return
      end
