subroutine gray_main(ijetto, mr, mz, r, z, psin, psiax, psibnd, &
    rax, zax, nbnd, rbnd, zbnd, nrho, psijet, f, te, dne, zeff, qsf, &
    beamid, powin, alphain, betain, dpdv, jcd, pec, icd, ierr)

  use const_and_precisions, only : r8
  use graydata_flags, only : igrad, ipass
  
  implicit none

  integer, intent(in) :: ijetto, mr, mz, nrho, nbnd, beamid
  real(r8), intent(in) :: r(mr), z(mz), psin(mr,mz)
  real(r8), intent(in) :: psiax, psibnd, rax, zax
  real(r8), intent(in), dimension(nbnd) :: rbnd, zbnd
  real(r8), intent(in), dimension(nrho) :: psijet, f, qsf, te, dne, zeff
  real(r8), intent(in) :: powin, alphain, betain
  real(r8), intent(out), dimension(nrho) :: dpdv, jcd
  real(r8), intent(out) :: pec, icd
  integer, intent(out) :: ierr

  integer :: istop, iercom, iopmin, iowmin, index_rt
  real(r8) :: sox, p0mw, powrfl, taumn, taumx, pabstot, currtot
  real(r8) :: p0mw1, powtr, pabstott, currtott
  real(r8), dimension(nrho) :: rhopol,dpdv1pass, jcd1pass

  common/istop/istop
  common/ierr/iercom
  common/iovmin/iopmin,iowmin
  common/mode/sox
  common/p0/p0mw
  common/powrfl/powrfl
  common/index_rt/index_rt
  common/taumnx/taumn,taumx,pabstot,currtot
  
! read data plus initialization
  index_rt=1
  call prfile
  call paraminit
  call read_data(ijetto, mr, mz, r, z, psin, psiax, psibnd, rax, zax, &
    nbnd, rbnd, zbnd, nrho, psijet, f, te, dne, zeff, qsf, powin, alphain, betain, &
    beamid)
  call vectinit
  if(iercom.eq.0) then
    if(igrad.eq.0) call ic_rt
    if(igrad.gt.0) call ic_gb
  end if
  if(iercom.gt.0) then
    ierr=iercom
    write(*,*) '    IERR = ', ierr
    return
  end if

! beam/ray propagation
  call gray_integration

! postprocessing
  rhopol=sqrt(psijet)
  call after_gray_integration(rhopol,nrho,dpdv1pass,jcd1pass)

  pabstott=pabstot
  currtott=currtot
  dpdv=dpdv1pass
  jcd=jcd1pass
  powtr=p0mw-pabstot

  if (iowmin==2 .and. ipass>1) then
! second pass into plasma
    p0mw1=p0mw
    igrad=0
    
    index_rt=2
    p0mw=p0mw1*powrfl
    call prfile
    call vectinit2
    call paraminit
    call ic_rt2
    call gray_integration
    call after_gray_integration(rhopol,nrho,dpdv1pass,jcd1pass)
    pabstott=pabstott+pabstot
    currtott=currtott+currtot
    dpdv=dpdv+dpdv1pass
    jcd=jcd+jcd1pass
  
    index_rt=3
    sox=-sox
    p0mw=p0mw1*(1._r8-powrfl)
    call prfile
    call vectinit2
    call paraminit
    call ic_rt2
    call gray_integration
    call after_gray_integration(rhopol,nrho,dpdv1pass,jcd1pass)
    pabstott=pabstott+pabstot
    currtott=currtott+currtot
    dpdv=dpdv+dpdv1pass
    jcd=jcd+jcd1pass
  end if
  write(*,*)
  write(*,*) 'Pabs (MW), Icd (kA) = ', pabstott,currtott*1.e3_r8
  call vectfree
  pec=pabstott*1.e6_r8
  icd=currtott*1.e6_r8
  dpdv=dpdv*1.e6_r8
  jcd=jcd*1.e6_r8
  ierr=iercom
end subroutine gray_main
